@@ -822,3 +822,16 @@ class AgentInProgress(IronicException):
 class InsufficentMemory(IronicException):
     _msg_fmt = _("Available memory at %(free)s, Insufficent as %(required)s "
                  "is required to proceed at this time.")
+
+
+class NodeHistoryNotFound(NotFound):
+    _msg_fmt = _("Node history record %(history)s could not be found.")
+
+
+class IncorrectConfiguration(IronicException):
+    _msg_fmt = _("Supplied configuration is incorrect and must be fixed. "
+                 "Error: %(error)s")
+
+
+class NodeVerifyFailure(IronicException):
+    _msg_fmt = _("Failed to verify node %(node)s: %(reason)s")
