@@ -128,18 +128,24 @@ opts = [
                help=_('Default boot option to use when no boot option is '
                       'requested in node\'s driver_info. Defaults to '
                       '"local". Prior to the Ussuri release, the default '
-                      'was "netboot".')),
+                      'was "netboot".'),
+               deprecated_for_removal=True,
+               deprecated_reason=_('Support for network boot will be removed '
+                                   'after the Yoga release.')),
     cfg.StrOpt('default_boot_mode',
                choices=[(boot_modes.UEFI, _('UEFI boot mode')),
                         (boot_modes.LEGACY_BIOS, _('Legacy BIOS boot mode'))],
-               default=boot_modes.LEGACY_BIOS,
+               default=boot_modes.UEFI,
                mutable=True,
                help=_('Default boot mode to use when no boot mode is '
                       'requested in node\'s driver_info, capabilities or '
                       'in the `instance_info` configuration. Currently the '
-                      'default boot mode is "%(bios)s", but it will be '
-                      'changed to "%(uefi)s in the future. It is recommended '
-                      'to set an explicit value for this option. This option '
+                      'default boot mode is "%(uefi)s", but it was '
+                      '"%(bios)s" previously in Ironic. It is recommended '
+                      'to set an explicit value for this option, and if the '
+                      'setting or default differs from nodes, to ensure that '
+                      'nodes are configured specifically for their desired '
+                      'boot mode. This option '
                       'only has effect when management interface supports '
                       'boot mode management') % {
                           'bios': boot_modes.LEGACY_BIOS,
@@ -201,7 +207,10 @@ opts = [
                         ('local', _('This is the default behavior. '
                                     'The image is downloaded, prepared and '
                                     'cached locally, to be served from '
-                                    'the conductor.'))],
+                                    'the conductor.')),
+                        ('swift', _('Same as "http", but if the image '
+                                    'is a Glance UUID, it is exposed via a '
+                                    'Swift temporary URL.'))],
                default='local',
                mutable=True,
                help=_('Specifies whether a boot iso image should be served '
@@ -209,6 +218,21 @@ opts = [
                       'url directly, or if ironic should cache the image on '
                       'the conductor and serve it from ironic\'s own http '
                       'server.')),
+    cfg.StrOpt('iso_master_path',
+               default='/var/lib/ironic/master_iso_images',
+               help=_('On the ironic-conductor node, directory where master '
+                      'ISO images are stored on disk. '
+                      'Setting to the empty string disables image caching.')),
+    cfg.IntOpt('iso_cache_size',
+               default=20480,
+               help=_('Maximum size (in MiB) of cache for master ISO images, '
+                      'including those in use.')),
+    # 10080 here is 1 week - 60*24*7. It is entirely arbitrary in the absence
+    # of a facility to disable the ttl entirely.
+    cfg.IntOpt('iso_cache_ttl',
+               default=10080,
+               help=_('Maximum TTL (in minutes) for old master ISO images in '
+                      'cache.')),
 ]
 
 
