@@ -13,20 +13,17 @@
 Base PXE Interface Methods
 """
 
-from futurist import periodics
 from ironic_lib import metrics_utils
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import strutils
 
 from ironic.common import boot_devices
 from ironic.common import dhcp_factory
 from ironic.common import exception
-from ironic.common.glance_service import service_utils
 from ironic.common.i18n import _
 from ironic.common import pxe_utils
 from ironic.common import states
-from ironic.conductor import task_manager
+from ironic.conductor import periodics
 from ironic.conductor import utils as manager_utils
 from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
@@ -45,18 +42,6 @@ REQUIRED_PROPERTIES = {
     'deploy_ramdisk': _("UUID (from Glance) of the ramdisk that is "
                         "mounted at boot time. Required."),
 }
-OPTIONAL_PROPERTIES = {
-    'force_persistent_boot_device': _("Controls the persistency of boot order "
-                                      "changes. 'Always' will make all "
-                                      "changes persistent, 'Default' will "
-                                      "make all but the final one upon "
-                                      "instance deployment non-persistent, "
-                                      "and 'Never' will make no persistent "
-                                      "changes at all. The old values 'True' "
-                                      "and 'False' are still supported but "
-                                      "deprecated in favor of the new ones."
-                                      "Defaults to 'Default'. Optional."),
-}
 RESCUE_PROPERTIES = {
     'rescue_kernel': _('UUID (from Glance) of the rescue kernel. This value '
                        'is required for rescue mode.'),
@@ -64,9 +49,19 @@ RESCUE_PROPERTIES = {
                         'that is used at node rescue time. This value is '
                         'required for rescue mode.'),
 }
+OPTIONAL_PROPERTIES = {
+    'kernel_append_params': _("Additional kernel parameters to pass down to "
+                              "instance kernel. These parameters can be "
+                              "consumed by the kernel or by the applications "
+                              "by reading /proc/cmdline. Mind severe cmdline "
+                              "size limit. Overrides "
+                              "[pxe]/kernel_append_params ironic "
+                              "option."),
+}
 COMMON_PROPERTIES = REQUIRED_PROPERTIES.copy()
-COMMON_PROPERTIES.update(OPTIONAL_PROPERTIES)
+COMMON_PROPERTIES.update(driver_utils.OPTIONAL_PROPERTIES)
 COMMON_PROPERTIES.update(RESCUE_PROPERTIES)
+COMMON_PROPERTIES.update(OPTIONAL_PROPERTIES)
 
 
 class PXEBaseMixin(object):
@@ -133,6 +128,8 @@ class PXEBaseMixin(object):
             pxe_utils.clean_up_pxe_env(task, images_info,
                                        ipxe_enabled=self.ipxe_enabled)
 
+        boot_mode_utils.deconfigure_secure_boot_if_needed(task)
+
     @METRICS.timer('PXEBaseMixin.prepare_ramdisk')
     def prepare_ramdisk(self, task, ramdisk_params):
         """Prepares the boot of Ironic ramdisk using PXE.
@@ -209,9 +206,8 @@ class PXEBaseMixin(object):
         pxe_utils.create_pxe_config(task, pxe_options,
                                     pxe_config_template,
                                     ipxe_enabled=self.ipxe_enabled)
-        persistent = self._persistent_ramdisk_boot(node)
         manager_utils.node_set_boot_device(task, boot_devices.PXE,
-                                           persistent=persistent)
+                                           persistent=False)
 
         if self.ipxe_enabled and CONF.pxe.ipxe_use_swift:
             kernel_label = '%s_kernel' % mode
@@ -240,23 +236,34 @@ class PXEBaseMixin(object):
         :returns: None
         """
         boot_mode_utils.sync_boot_mode(task)
+        boot_mode_utils.configure_secure_boot_if_needed(task)
 
         node = task.node
         boot_option = deploy_utils.get_boot_option(node)
         boot_device = None
         instance_image_info = {}
-        if boot_option == "ramdisk":
+        if boot_option == "ramdisk" or boot_option == "kickstart":
             instance_image_info = pxe_utils.get_instance_image_info(
                 task, ipxe_enabled=self.ipxe_enabled)
             pxe_utils.cache_ramdisk_kernel(task, instance_image_info,
                                            ipxe_enabled=self.ipxe_enabled)
+            if 'ks_template' in instance_image_info:
+                ks_cfg = pxe_utils.validate_kickstart_template(
+                    instance_image_info['ks_template'][1]
+                )
+                pxe_utils.validate_kickstart_file(ks_cfg)
 
-        if deploy_utils.is_iscsi_boot(task) or boot_option == "ramdisk":
+        if (deploy_utils.is_iscsi_boot(task) or boot_option == "ramdisk"
+                or boot_option == "kickstart"):
             pxe_utils.prepare_instance_pxe_config(
                 task, instance_image_info,
                 iscsi_boot=deploy_utils.is_iscsi_boot(task),
                 ramdisk_boot=(boot_option == "ramdisk"),
+                anaconda_boot=(boot_option == "kickstart"),
                 ipxe_enabled=self.ipxe_enabled)
+            pxe_utils.prepare_instance_kickstart_config(
+                task, instance_image_info,
+                anaconda_boot=(boot_option == "kickstart"))
             boot_device = boot_devices.PXE
 
         elif boot_option != "local":
@@ -325,26 +332,37 @@ class PXEBaseMixin(object):
         # NOTE(pas-ha) do not re-set boot device on ACTIVE nodes
         # during takeover
         if boot_device and task.node.provision_state != states.ACTIVE:
-            persistent = True
-            if node.driver_info.get('force_persistent_boot_device',
-                                    'Default') == 'Never':
-                persistent = False
             manager_utils.node_set_boot_device(task, boot_device,
-                                               persistent=persistent)
+                                               persistent=True)
 
     def _validate_common(self, task):
         node = task.node
 
         if not driver_utils.get_node_mac_addresses(task):
             raise exception.MissingParameterValue(
-                _("Node %s does not have any port associated with it.")
+                _("Node %s does not have any port associated with it")
                 % node.uuid)
 
         if self.ipxe_enabled:
             if not CONF.deploy.http_url or not CONF.deploy.http_root:
                 raise exception.MissingParameterValue(_(
                     "iPXE boot is enabled but no HTTP URL or HTTP "
-                    "root was specified."))
+                    "root was specified"))
+
+        # NOTE(zer0c00l): When 'kickstart' boot option is used we need to store
+        # kickstart and squashfs files in http_root directory. These files
+        # will be eventually requested by anaconda installer during deployment
+        # over http(s).
+        if deploy_utils.get_boot_option(node) == 'kickstart':
+            if not CONF.deploy.http_url or not CONF.deploy.http_root:
+                raise exception.MissingParameterValue(_(
+                    "'kickstart' boot option is set on the node but no HTTP "
+                    "URL or HTTP root was specified"))
+
+            if not CONF.anaconda.default_ks_template:
+                raise exception.MissingParameterValue(_(
+                    "'kickstart' boot option is set on the node but no "
+                    "default kickstart template is specified"))
 
         # Check the trusted_boot capabilities value.
         deploy_utils.validate_capabilities(node)
@@ -371,7 +389,7 @@ class PXEBaseMixin(object):
             and node.instance_info.get('boot_iso')):
             raise exception.InvalidParameterValue(_(
                 "An 'image_source' and 'boot_iso' parameter may not be "
-                "specified at the same time."))
+                "specified at the same time"))
 
         pxe_utils.parse_driver_info(node)
 
@@ -390,24 +408,20 @@ class PXEBaseMixin(object):
             missing.
         """
         self._validate_common(task)
+        node = task.node
 
         # NOTE(TheJulia): If we're not writing an image, we can skip
         # the remainder of this method.
-        if (not task.driver.storage.should_write_image(task)):
+        # NOTE(dtantsur): if we're are writing an image with local boot
+        # the boot interface does not care about image parameters and
+        # must not validate them.
+        boot_option = deploy_utils.get_boot_option(node)
+        if (not task.driver.storage.should_write_image(task)
+                or boot_option == 'local'):
             return
 
-        node = task.node
         d_info = deploy_utils.get_image_instance_info(node)
-        if (node.driver_internal_info.get('is_whole_disk_image')
-                or deploy_utils.get_boot_option(node) == 'local'):
-            props = []
-        elif d_info.get('boot_iso'):
-            props = ['boot_iso']
-        elif service_utils.is_glance_image(d_info['image_source']):
-            props = ['kernel_id', 'ramdisk_id']
-        else:
-            props = ['kernel', 'ramdisk']
-        deploy_utils.validate_image_properties(task.context, d_info, props)
+        deploy_utils.validate_image_properties(task, d_info)
 
     @METRICS.timer('PXEBaseMixin.validate_rescue')
     def validate_rescue(self, task):
@@ -433,46 +447,29 @@ class PXEBaseMixin(object):
             raise exception.UnsupportedDriverExtension(
                 driver=task.node.driver, extension='inspection')
 
-    def _persistent_ramdisk_boot(self, node):
-        """If the ramdisk should be configured as a persistent boot device."""
-        value = node.driver_info.get('force_persistent_boot_device', 'Default')
-        if value in {'Always', 'Default', 'Never'}:
-            return value == 'Always'
-        else:
-            return strutils.bool_from_string(value, False)
-
     _RETRY_ALLOWED_STATES = {states.DEPLOYWAIT, states.CLEANWAIT,
                              states.RESCUEWAIT}
 
     @METRICS.timer('PXEBaseMixin._check_boot_timeouts')
-    @periodics.periodic(spacing=CONF.pxe.boot_retry_check_interval,
-                        enabled=bool(CONF.pxe.boot_retry_timeout))
-    def _check_boot_timeouts(self, manager, context):
+    @periodics.node_periodic(
+        purpose='checking PXE boot status',
+        spacing=CONF.pxe.boot_retry_check_interval,
+        enabled=bool(CONF.pxe.boot_retry_timeout),
+        filters={'provision_state_in': _RETRY_ALLOWED_STATES,
+                 'reserved': False,
+                 'maintenance': False,
+                 'provisioned_before': CONF.pxe.boot_retry_timeout},
+    )
+    def _check_boot_timeouts(self, task, manager, context):
         """Periodically checks whether boot has timed out and retry it.
 
+        :param task: a task instance.
         :param manager: conductor manager.
         :param context: request context.
         """
-        filters = {'provision_state_in': self._RETRY_ALLOWED_STATES,
-                   'reserved': False,
-                   'maintenance': False,
-                   'provisioned_before': CONF.pxe.boot_retry_timeout}
-        node_iter = manager.iter_nodes(filters=filters)
-
-        for node_uuid, driver, conductor_group in node_iter:
-            try:
-                lock_purpose = 'checking PXE boot status'
-                with task_manager.acquire(context, node_uuid,
-                                          shared=True,
-                                          purpose=lock_purpose) as task:
-                    self._check_boot_status(task)
-            except (exception.NodeLocked, exception.NodeNotFound):
-                continue
+        self._check_boot_status(task)
 
     def _check_boot_status(self, task):
-        if not isinstance(task.driver.boot, PXEBaseMixin):
-            return
-
         if not _should_retry_boot(task.node):
             return
 
@@ -490,17 +487,20 @@ class PXEBaseMixin(object):
                   'timeout': CONF.pxe.boot_retry_timeout})
 
         manager_utils.node_power_action(task, states.POWER_OFF)
-        # NOTE(dtantsur): retry even persistent boot setting in case it did not
-        # work for some reason.
-        persistent = self._persistent_ramdisk_boot(task.node)
         manager_utils.node_set_boot_device(task, boot_devices.PXE,
-                                           persistent=persistent)
+                                           persistent=False)
         manager_utils.node_power_action(task, states.POWER_ON)
 
 
 def _should_retry_boot(node):
     # NOTE(dtantsur): this assumes IPA, do we need to make it generic?
     for field in ('agent_last_heartbeat', 'last_power_state_change'):
+        if node.driver_internal_info.get('agent_secret_token', False):
+            LOG.debug('Not retrying PXE boot for node %(node)s; an agent '
+                      'token has been identified, meaning the agent '
+                      'has started.',
+                      {'node': node.uuid})
+            return False
         if manager_utils.value_within_timeout(
                 node.driver_internal_info.get(field),
                 CONF.pxe.boot_retry_timeout):
