@@ -15,10 +15,10 @@
 
 from oslo_log import log
 from oslo_utils import importutils
+import tenacity
 
 from ironic.common import boot_devices
 from ironic.common import exception
-from ironic.common.glance_service import service_utils
 from ironic.common.i18n import _
 from ironic.common import states
 from ironic.conductor import utils as manager_utils
@@ -28,21 +28,24 @@ from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import image_utils
 from ironic.drivers.modules.redfish import utils as redfish_utils
+from ironic.drivers import utils as driver_utils
 
 LOG = log.getLogger(__name__)
 
 REQUIRED_PROPERTIES = {
     'deploy_kernel': _("URL or Glance UUID of the deployment kernel. "
-                       "Required."),
-    'deploy_ramdisk': _("URL or Glance UUID of the ramdisk that is "
-                        "mounted at boot time. Required.")
+                       "Required if deploy_iso is not set."),
+    'deploy_ramdisk': _("URL or Glance UUID of the ramdisk that is mounted at "
+                        "boot time. Required if deploy_iso is not set."),
+    'deploy_iso': _("URL or Glance UUID of the deployment ISO to use. "
+                    "Required if deploy_kernel/deploy_ramdisk are not set.")
 }
 
 OPTIONAL_PROPERTIES = {
-    'config_via_floppy': _("Boolean value to indicate whether or not the "
-                           "driver should use virtual media Floppy device "
-                           "for passing configuration information to the "
-                           "ramdisk. Defaults to False. Optional."),
+    'config_via_removable': _("Boolean value to indicate whether or not the "
+                              "driver should use virtual media USB or floppy "
+                              "device for passing configuration information "
+                              "to the ramdisk. Defaults to False. Optional."),
     'kernel_append_params': _("Additional kernel parameters to pass down to "
                               "instance kernel. These parameters can be "
                               "consumed by the kernel or by the applications "
@@ -54,27 +57,26 @@ OPTIONAL_PROPERTIES = {
                     "image containing EFI boot loader. This image will be "
                     "used by ironic when building UEFI-bootable ISO "
                     "out of kernel and ramdisk. Required for UEFI "
-                    "boot from partition images."),
+                    "when deploy_iso is not provided."),
 
 }
 
 RESCUE_PROPERTIES = {
-    'rescue_kernel': _('URL or Glance UUID of the rescue kernel. This value '
-                       'is required for rescue mode.'),
+    'rescue_kernel': _('URL or Glance UUID of the rescue kernel. Required for '
+                       'rescue mode if rescue_iso is not set.'),
     'rescue_ramdisk': _('URL or Glance UUID of the rescue ramdisk with agent '
-                        'that is used at node rescue time. This value is '
-                        'required for rescue mode.'),
+                        'that is used at node rescue time. Required for '
+                        'rescue mode if rescue_iso is not set.'),
+    'rescue_iso': _("URL or Glance UUID of the rescue ISO to use. Required "
+                    "for rescue mode if rescue_kernel/rescue_ramdisk are "
+                    "not set.")
 }
 
 COMMON_PROPERTIES = REQUIRED_PROPERTIES.copy()
+COMMON_PROPERTIES.update(driver_utils.OPTIONAL_PROPERTIES)
 COMMON_PROPERTIES.update(OPTIONAL_PROPERTIES)
 COMMON_PROPERTIES.update(RESCUE_PROPERTIES)
 
-KERNEL_RAMDISK_LABELS = {
-    'deploy': REQUIRED_PROPERTIES,
-    'rescue': RESCUE_PROPERTIES
-}
-
 IMAGE_SUBDIR = 'redfish'
 
 sushy = importutils.try_import('sushy')
@@ -94,31 +96,46 @@ def _parse_driver_info(node):
     :raises: InvalidParameterValue, if any of the parameters have invalid
         value.
     """
+    mode = deploy_utils.rescue_or_deploy_mode(node)
+    if not deploy_utils.needs_agent_ramdisk(node, mode=mode):
+        # Ramdisk deploy does not need an agent, nor does it support any other
+        # options. Skipping.
+        return {'can_provide_config': False}
+
     d_info = node.driver_info
 
-    mode = deploy_utils.rescue_or_deploy_mode(node)
-    params_to_check = KERNEL_RAMDISK_LABELS[mode]
+    iso_param = f'{mode}_iso'
+    iso_ref = driver_utils.get_agent_iso(node, deprecated_prefix='redfish',
+                                         mode=mode)
+    if iso_ref is not None:
+        deploy_info = {iso_param: iso_ref}
+        can_config = False
+    else:
+        # There was never a deprecated prefix for kernel/ramdisk
+        deploy_info = driver_utils.get_agent_kernel_ramdisk(node, mode)
 
-    deploy_info = {option: d_info.get(option)
-                   for option in params_to_check}
+        error_msg = _("Error validating Redfish virtual media. Some "
+                      "parameters were missing in node's driver_info")
 
-    if not any(deploy_info.values()):
-        # NOTE(dtantsur): avoid situation when e.g. deploy_kernel comes
-        # from driver_info but deploy_ramdisk comes from configuration,
-        # since it's a sign of a potential operator's mistake.
-        deploy_info = {k: getattr(CONF.conductor, k)
-                       for k in params_to_check}
-
-    error_msg = _("Error validating Redfish virtual media. Some "
-                  "parameters were missing in node's driver_info")
-
-    deploy_utils.check_for_missing_params(deploy_info, error_msg)
+        deploy_utils.check_for_missing_params(deploy_info, error_msg)
+        can_config = True
 
     deploy_info.update(
         {option: d_info.get(option, getattr(CONF.conductor, option, None))
          for option in OPTIONAL_PROPERTIES})
 
+    if (d_info.get('config_via_removable') is None
+            and d_info.get('config_via_floppy') is not None):
+        LOG.warning('The config_via_floppy driver_info option is deprecated, '
+                    'use config_via_removable for node %s', node.uuid)
+        deploy_info['config_via_removable'] = d_info['config_via_floppy']
+
     deploy_info.update(redfish_utils.parse_driver_info(node))
+    # Configuration can be provided in one of two cases:
+    # 1) A removable disk is requested.
+    # 2) An ISO is built from a kernel/initramfs pair.
+    deploy_info['can_provide_config'] = \
+        deploy_info.get('config_via_removable') or can_config
 
     return deploy_info
 
@@ -141,19 +158,33 @@ def _parse_instance_info(node):
     return deploy_info
 
 
-def _insert_vmedia(task, boot_url, boot_device):
+def _test_retry(exception):
+    if isinstance(exception, sushy.exceptions.ServerSideError):
+        # On some Dell hw, the eject media may still be in progress
+        # https://storyboard.openstack.org/#!/story/2008504
+        LOG.warning("Boot media insert failed for node %(node)s, "
+                    "will retry after 3 seconds",
+                    {'node': exception.node_uuid})
+        return True
+    return False
+
+
+@tenacity.retry(retry=tenacity.retry_if_exception(_test_retry),
+                stop=tenacity.stop_after_attempt(3),
+                wait=tenacity.wait_fixed(3),
+                reraise=True)
+def _insert_vmedia(task, managers, boot_url, boot_device):
     """Insert bootable ISO image into virtual CD or DVD
 
     :param task: A task from TaskManager.
+    :param managers: A list of System managers.
     :param boot_url: URL to a bootable ISO image
     :param boot_device: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
         `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY`
     :raises: InvalidParameterValue, if no suitable virtual CD or DVD is
         found on the node.
     """
-    system = redfish_utils.get_system(task.node)
-
-    for manager in system.managers:
+    for manager in managers:
         for v_media in manager.virtual_media.get_members():
             if boot_device not in v_media.media_types:
                 continue
@@ -169,8 +200,12 @@ def _insert_vmedia(task, boot_url, boot_device):
 
                 continue
 
-            v_media.insert_media(boot_url, inserted=True,
-                                 write_protected=True)
+            try:
+                v_media.insert_media(boot_url, inserted=True,
+                                     write_protected=True)
+            except sushy.exceptions.ServerSideError as e:
+                e.node_uuid = task.node.uuid
+                raise
 
             LOG.info("Inserted boot media %(boot_url)s into "
                      "%(boot_device)s for node "
@@ -183,7 +218,39 @@ def _insert_vmedia(task, boot_url, boot_device):
         _('No suitable virtual media device found'))
 
 
-def _eject_vmedia(task, boot_device=None):
+def _eject_vmedia(task, managers, boot_device=None):
+    """Eject virtual CDs and DVDs
+
+    :param task: A task from TaskManager.
+    :param managers: A list of System managers.
+    :param boot_device: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
+        `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY` or `None` to
+        eject everything (default).
+    :return: True if any device was ejected, else False
+    :raises: InvalidParameterValue, if no suitable virtual CD or DVD is
+        found on the node.
+    """
+    found = False
+    for manager in managers:
+        for v_media in manager.virtual_media.get_members():
+            if boot_device and boot_device not in v_media.media_types:
+                continue
+
+            inserted = v_media.inserted
+
+            if inserted:
+                v_media.eject_media()
+                found = True
+
+            LOG.info("Boot media is%(already)s ejected from "
+                     "%(boot_device)s for node %(node)s"
+                     "", {'node': task.node.uuid,
+                          'already': '' if inserted else ' already',
+                          'boot_device': v_media.name})
+    return found
+
+
+def eject_vmedia(task, boot_device=None):
     """Eject virtual CDs and DVDs
 
     :param task: A task from TaskManager.
@@ -194,37 +261,41 @@ def _eject_vmedia(task, boot_device=None):
         found on the node.
     """
     system = redfish_utils.get_system(task.node)
-
-    for manager in system.managers:
-        for v_media in manager.virtual_media.get_members():
-            if boot_device and boot_device not in v_media.media_types:
-                continue
-
-            inserted = v_media.inserted
-
-            if inserted:
-                v_media.eject_media()
-
-            LOG.info("Boot media is%(already)s ejected from "
-                     "%(boot_device)s for node %(node)s"
-                     "", {'node': task.node.uuid,
-                          'already': '' if inserted else ' already',
-                          'boot_device': v_media.name})
+    if _eject_vmedia(task, system.managers, boot_device=boot_device):
+        LOG.debug('Cleaning up unused files after ejecting %(dev)s for node '
+                  '%(node)s', {'dev': boot_device or 'all devices',
+                               'node': task.node.uuid})
+        if (boot_device is None
+                or boot_device == sushy.VIRTUAL_MEDIA_USBSTICK):
+            image_utils.cleanup_disk_image(task, prefix='configdrive')
+        if boot_device is None or boot_device == sushy.VIRTUAL_MEDIA_CD:
+            image_utils.cleanup_iso_image(task)
 
 
-def _has_vmedia_device(task, boot_device):
+def _has_vmedia_device(managers, boot_device, inserted=None):
     """Indicate if device exists at any of the managers
 
-    :param task: A task from TaskManager.
-    :param boot_device: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
-        `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY`.
+    :param managers: A list of System managers.
+    :param boot_device: One or more sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
+        `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY`. Several devices are
+        checked in the given order.
+    :param inserted: If not None, only return a device with a matching
+        inserted status.
+    :return: The device that could be found or False.
     """
-    system = redfish_utils.get_system(task.node)
+    if not isinstance(boot_device, list):
+        boot_device = [boot_device]
 
-    for manager in system.managers:
-        for v_media in manager.virtual_media.get_members():
-            if boot_device in v_media.media_types:
-                return True
+    for dev in boot_device:
+        for manager in managers:
+            for v_media in manager.virtual_media.get_members():
+                if dev not in v_media.media_types:
+                    continue
+                if (inserted is not None
+                        and bool(v_media.inserted) is not inserted):
+                    continue
+                return dev
+    return False
 
 
 def _parse_deploy_info(node):
@@ -279,7 +350,8 @@ class RedfishVirtualMediaBoot(base.BootInterface):
     `[instance_info]image_source` node property.
     """
 
-    capabilities = ['iscsi_volume_boot', 'ramdisk_boot']
+    capabilities = ['iscsi_volume_boot', 'ramdisk_boot',
+                    'ramdisk_boot_configdrive']
 
     def __init__(self):
         """Initialize the Redfish virtual media boot interface.
@@ -314,6 +386,8 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         node = task.node
 
         _parse_driver_info(node)
+        # Issue the deprecation warning if needed
+        driver_utils.get_agent_iso(node, deprecated_prefix='redfish')
 
     def _validate_instance_info(self, task):
         """Validate instance image information for the task's node.
@@ -328,19 +402,44 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         """
         node = task.node
 
+        # NOTE(dtantsur): if we're are writing an image with local boot
+        # the boot interface does not care about image parameters and
+        # must not validate them.
+        if (not task.driver.storage.should_write_image(task)
+                or deploy_utils.get_boot_option(node) == 'local'):
+            return
+
         d_info = _parse_deploy_info(node)
+        deploy_utils.validate_image_properties(task, d_info)
 
-        if node.driver_internal_info.get('is_whole_disk_image'):
-            props = []
-        elif d_info.get('boot_iso'):
-            props = ['boot_iso']
-        elif service_utils.is_glance_image(d_info['image_source']):
-            props = ['kernel_id', 'ramdisk_id']
+    def _validate_vendor(self, task, managers):
+        """Validates vendor specific requirements for the task's node.
 
-        else:
-            props = ['kernel', 'ramdisk']
+        :param task: a TaskManager instance containing the node to act on.
+        :param managers: Redfish managers for Redfish system associated
+            with node.
+        :raises: InvalidParameterValue if vendor not supported
+        """
+        vendor = task.node.properties.get('vendor')
+        if not vendor:
+            return
 
-        deploy_utils.validate_image_properties(task.context, d_info, props)
+        if 'Dell' in vendor.split():
+            # Check if iDRAC fw >= 6.00.00.00 that supports virtual media boot
+            bmc_manager = [m for m in managers
+                           if m.manager_type == sushy.MANAGER_TYPE_BMC]
+            if bmc_manager:
+                fwv = bmc_manager[0].firmware_version.split('.')
+                if int(fwv[0]) >= 6:
+                    return
+            raise exception.InvalidParameterValue(
+                _("The %(iface)s boot interface is not suitable for node "
+                  "%(node)s with vendor %(vendor)s and BMC version %(fwv)s, "
+                  "upgrade to 6.00.00.00 or newer or use "
+                  "idrac-redfish-virtual-media instead")
+                % {'iface': task.node.get_interface('boot'),
+                   'node': task.node.uuid, 'vendor': vendor,
+                   'fwv': bmc_manager[0].firmware_version})
 
     def validate(self, task):
         """Validate the deployment information for the task's node.
@@ -354,9 +453,7 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         :raises: MissingParameterValue on missing parameter(s)
         """
         self._validate_driver_info(task)
-
-        if task.driver.storage.should_write_image(task):
-            self._validate_instance_info(task)
+        self._validate_instance_info(task)
 
     def validate_inspection(self, task):
         """Validate that the node has required properties for inspection.
@@ -391,64 +488,81 @@ class RedfishVirtualMediaBoot(base.BootInterface):
             operation failed on the node.
         """
         node = task.node
-        # NOTE(TheJulia): If this method is being called by something
-        # aside from deployment, clean and rescue, such as conductor takeover,
-        # we should treat this as a no-op and move on otherwise we would
-        # modify the state of the node due to virtual media operations.
-        if node.provision_state not in (states.DEPLOYING,
-                                        states.CLEANING,
-                                        states.RESCUING,
-                                        states.INSPECTING):
+        if not driver_utils.need_prepare_ramdisk(node):
             return
 
+        d_info = _parse_driver_info(node)
+        managers = redfish_utils.get_system(task.node).managers
+
+        self._validate_vendor(task, managers)
+
+        if manager_utils.is_fast_track(task):
+            if _has_vmedia_device(managers, sushy.VIRTUAL_MEDIA_CD,
+                                  inserted=True):
+                LOG.debug('Fast track operation for node %s, not inserting '
+                          'any devices', node.uuid)
+                return
+            else:
+                LOG.warning('Fast track is possible for node %s, but no ISO '
+                            'is currently inserted! Proceeding with '
+                            'normal operation.', node.uuid)
+
         # NOTE(TheJulia): Since we're deploying, cleaning, or rescuing,
         # with virtual media boot, we should generate a token!
-        manager_utils.add_secret_token(node, pregenerated=True)
-        node.save()
-        ramdisk_params['ipa-agent-token'] = \
-            node.driver_internal_info['agent_secret_token']
+        # However, we don't have a way to inject it with a pre-built ISO
+        # if a removable disk is not used.
+        can_config = d_info.pop('can_provide_config', True)
+        if can_config:
+            manager_utils.add_secret_token(node, pregenerated=True)
+            node.save()
+            ramdisk_params['ipa-agent-token'] = \
+                node.driver_internal_info['agent_secret_token']
 
         manager_utils.node_power_action(task, states.POWER_OFF)
 
-        d_info = _parse_driver_info(node)
-
-        config_via_floppy = d_info.get('config_via_floppy')
-
         deploy_nic_mac = deploy_utils.get_single_nic_with_vif_port_id(task)
         if deploy_nic_mac is not None:
             ramdisk_params['BOOTIF'] = deploy_nic_mac
         if CONF.debug and 'ipa-debug' not in ramdisk_params:
             ramdisk_params['ipa-debug'] = '1'
 
-        if config_via_floppy:
+        # NOTE(TheJulia): This is a mandatory setting for virtual media
+        # based deployment operations.
+        ramdisk_params['boot_method'] = 'vmedia'
 
-            if _has_vmedia_device(task, sushy.VIRTUAL_MEDIA_FLOPPY):
-                # NOTE (etingof): IPA will read the diskette only if
-                # we tell it to
-                ramdisk_params['boot_method'] = 'vmedia'
+        config_via_removable = d_info.get('config_via_removable')
+        if config_via_removable:
 
+            removable = _has_vmedia_device(
+                managers,
+                # Prefer USB devices since floppies are outdated
+                [sushy.VIRTUAL_MEDIA_USBSTICK, sushy.VIRTUAL_MEDIA_FLOPPY])
+            if removable:
                 floppy_ref = image_utils.prepare_floppy_image(
                     task, params=ramdisk_params)
 
-                _eject_vmedia(task, sushy.VIRTUAL_MEDIA_FLOPPY)
-                _insert_vmedia(
-                    task, floppy_ref, sushy.VIRTUAL_MEDIA_FLOPPY)
+                _eject_vmedia(task, managers, removable)
+                _insert_vmedia(task, managers, floppy_ref, removable)
 
-                LOG.debug('Inserted virtual floppy with configuration for '
-                          'node %(node)s', {'node': task.node.uuid})
+                LOG.info('Inserted virtual %(type)s device with configuration'
+                         ' for node %(node)s',
+                         {'node': task.node.uuid, 'type': removable})
 
             else:
-                LOG.warning('Config via floppy is requested, but '
-                            'Floppy drive is not available on node '
-                            '%(node)s', {'node': task.node.uuid})
+                LOG.warning('Config via a removable device is requested, but '
+                            'virtual USB and floppy devices are not '
+                            'available on node %(node)s',
+                            {'node': task.node.uuid})
 
         mode = deploy_utils.rescue_or_deploy_mode(node)
 
         iso_ref = image_utils.prepare_deploy_iso(task, ramdisk_params,
                                                  mode, d_info)
 
-        _eject_vmedia(task, sushy.VIRTUAL_MEDIA_CD)
-        _insert_vmedia(task, iso_ref, sushy.VIRTUAL_MEDIA_CD)
+        _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_CD)
+        _insert_vmedia(task, managers, iso_ref, sushy.VIRTUAL_MEDIA_CD)
+
+        del managers
 
         boot_mode_utils.sync_boot_mode(task)
 
@@ -467,21 +581,14 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         :param task: A task from TaskManager.
         :returns: None
         """
-        d_info = _parse_driver_info(task.node)
-
-        config_via_floppy = d_info.get('config_via_floppy')
+        if manager_utils.is_fast_track(task):
+            LOG.debug('Fast track operation for node %s, not ejecting '
+                      'any devices', task.node.uuid)
+            return
 
         LOG.debug("Cleaning up deploy boot for "
                   "%(node)s", {'node': task.node.uuid})
-
-        _eject_vmedia(task, sushy.VIRTUAL_MEDIA_CD)
-        image_utils.cleanup_iso_image(task)
-
-        if (config_via_floppy
-                and _has_vmedia_device(task, sushy.VIRTUAL_MEDIA_FLOPPY)):
-            _eject_vmedia(task, sushy.VIRTUAL_MEDIA_FLOPPY)
-
-            image_utils.cleanup_floppy_image(task)
+        self._eject_all(task)
 
     def prepare_instance(self, task):
         """Prepares the boot of instance over virtual media.
@@ -505,10 +612,12 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         """
         node = task.node
 
+        self._eject_all(task)
+
         boot_mode_utils.sync_boot_mode(task)
+        boot_mode_utils.configure_secure_boot_if_needed(task)
 
         boot_option = deploy_utils.get_boot_option(node)
-        self.clean_up_instance(task)
         iwdi = node.driver_internal_info.get('is_whole_disk_image')
         if boot_option == "local" or iwdi:
             self._set_boot_device(task, boot_devices.DISK, persistent=True)
@@ -533,10 +642,17 @@ class RedfishVirtualMediaBoot(base.BootInterface):
 
             params.update(root_uuid=root_uuid)
 
+        managers = redfish_utils.get_system(task.node).managers
+
         deploy_info = _parse_deploy_info(node)
         iso_ref = image_utils.prepare_boot_iso(task, deploy_info, **params)
-        _eject_vmedia(task, sushy.VIRTUAL_MEDIA_CD)
-        _insert_vmedia(task, iso_ref, sushy.VIRTUAL_MEDIA_CD)
+        _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_CD)
+        _insert_vmedia(task, managers, iso_ref, sushy.VIRTUAL_MEDIA_CD)
+
+        if boot_option == 'ramdisk':
+            self._attach_configdrive(task, managers)
+
+        del managers
 
         self._set_boot_device(task, boot_devices.CDROM, persistent=True)
 
@@ -544,6 +660,43 @@ class RedfishVirtualMediaBoot(base.BootInterface):
                   "%(device)s", {'node': task.node.uuid,
                                  'device': boot_devices.CDROM})
 
+    def _attach_configdrive(self, task, managers):
+        configdrive = manager_utils.get_configdrive_image(task.node)
+        if not configdrive:
+            return
+
+        _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_USBSTICK)
+        cd_ref = image_utils.prepare_configdrive_image(task, configdrive)
+        try:
+            _insert_vmedia(task, managers, cd_ref,
+                           sushy.VIRTUAL_MEDIA_USBSTICK)
+        except exception.InvalidParameterValue:
+            raise exception.InstanceDeployFailure(
+                _('Cannot attach configdrive for node %s: no suitable '
+                  'virtual USB slot has been found') % task.node.uuid)
+
+    def _eject_all(self, task):
+        managers = redfish_utils.get_system(task.node).managers
+
+        _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_CD)
+        config_via_removable = (
+            task.node.driver_info.get('config_via_removable')
+            or task.node.driver_info.get('config_via_floppy')
+        )
+        if config_via_removable:
+            _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_USBSTICK)
+            _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_FLOPPY)
+
+            image_utils.cleanup_floppy_image(task)
+
+        boot_option = deploy_utils.get_boot_option(task.node)
+        if (boot_option == 'ramdisk'
+                and task.node.instance_info.get('configdrive') is not None):
+            _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_USBSTICK)
+            image_utils.cleanup_disk_image(task, prefix='configdrive')
+
+        image_utils.cleanup_iso_image(task)
+
     def clean_up_instance(self, task):
         """Cleans up the boot of instance.
 
@@ -555,14 +708,8 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         """
         LOG.debug("Cleaning up instance boot for "
                   "%(node)s", {'node': task.node.uuid})
-
-        _eject_vmedia(task, sushy.VIRTUAL_MEDIA_CD)
-        d_info = task.node.driver_info
-        config_via_floppy = d_info.get('config_via_floppy')
-        if config_via_floppy:
-            _eject_vmedia(task, sushy.VIRTUAL_MEDIA_FLOPPY)
-
-        image_utils.cleanup_iso_image(task)
+        self._eject_all(task)
+        boot_mode_utils.deconfigure_secure_boot_if_needed(task)
 
     @classmethod
     def _set_boot_device(cls, task, device, persistent=False):
