@@ -29,7 +29,6 @@ import time
 from oslo_log import log as logging
 from oslo_service import loopingcall
 from oslo_utils import importutils
-import six
 
 from ironic.common import exception
 from ironic.common.i18n import _
@@ -231,7 +230,10 @@ class SNMPClient(object):
             object depending on SNMP version being used.
         """
         if self.version == SNMP_V3:
-            return snmp.UsmUserData(
+            # NOTE(TheJulia): Ignore Bandit error B509 argument parsing as
+            # the check is for a count of <3 arguments, however our line
+            # wrapping causes the check to trigger.
+            return snmp.UsmUserData(  # nosec B509
                 self.user,
                 authKey=self.auth_key,
                 authProtocol=self.auth_proto,
@@ -405,7 +407,7 @@ def memoize(f):
     def memoized(self, node_info):
         hashable_node_info = frozenset((key, val)
                                        for key, val in node_info.items()
-                                       if key is not 'outlet')
+                                       if key != 'outlet')
         if hashable_node_info not in _memoized:
             _memoized[hashable_node_info] = f(self)
         return _memoized[hashable_node_info]
@@ -421,7 +423,7 @@ def retry_on_outdated_cache(f):
             hashable_node_info = (
                 frozenset((key, val)
                           for key, val in self.snmp_info.items()
-                          if key is not 'outlet')
+                          if key != 'outlet')
             )
             del _memoized[hashable_node_info]
             self.driver = self._get_pdu_driver(self.snmp_info)
@@ -430,8 +432,7 @@ def retry_on_outdated_cache(f):
     return wrapper
 
 
-@six.add_metaclass(abc.ABCMeta)
-class SNMPDriverBase(object):
+class SNMPDriverBase(object, metaclass=abc.ABCMeta):
     """SNMP power driver base class.
 
     The SNMPDriver class hierarchy implements manufacturer-specific MIB actions
@@ -513,6 +514,7 @@ class SNMPDriverBase(object):
         :raises: SNMPFailure if an SNMP request fails.
         :returns: power state. One of :class:`ironic.common.states`.
         """
+        time.sleep(CONF.snmp.power_action_delay)
         self._snmp_power_on()
         return self._snmp_wait_for_state(states.POWER_ON)
 
@@ -523,6 +525,7 @@ class SNMPDriverBase(object):
         :returns: power state. One of :class:`ironic.common.states`.
         """
         self._snmp_power_off()
+        time.sleep(CONF.snmp.power_action_delay)
         return self._snmp_wait_for_state(states.POWER_OFF)
 
     def power_reset(self):
@@ -556,15 +559,18 @@ class SNMPDriverSimple(SNMPDriverBase):
         super(SNMPDriverSimple, self).__init__(*args, **kwargs)
         self.oid = self._snmp_oid()
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def oid_device(self):
         """Device dependent portion of the power state object OID."""
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def value_power_on(self):
         """Value representing power on state."""
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def value_power_off(self):
         """Value representing power off state."""
 
@@ -810,8 +816,8 @@ class SNMPDriverAuto(SNMPDriverBase):
 
             system_id = self.oid_enterprise + getattr(obj, 'system_id')
 
-            if (system_id in drivers_map and
-                    drivers_map[system_id] is not obj):
+            if (system_id in drivers_map
+                    and drivers_map[system_id] is not obj):
                 raise exception.InvalidParameterValue(_(
                     "SNMPDriverAuto: duplicate driver system ID prefix "
                     "%(system_id)s") % {'system_id': system_id})
@@ -860,6 +866,7 @@ class SNMPDriverAuto(SNMPDriverBase):
     def _fetch_driver(self):
         return self.client.get(self.SYS_OBJ_OID)
 
+
 # A dictionary of supported drivers keyed by snmp_driver attribute
 DRIVER_CLASSES = {
     'apc': SNMPDriverAPCMasterSwitch,
@@ -955,26 +962,26 @@ def _parse_driver_info_snmpv3_crypto(node, info):
         if 'priv_protocol' not in snmp_info:
             snmp_info['priv_protocol'] = snmp_priv_protocols['des']
 
-    if ('priv_protocol' in snmp_info and
-            'auth_protocol' not in snmp_info):
+    if ('priv_protocol' in snmp_info
+            and 'auth_protocol' not in snmp_info):
         raise exception.MissingParameterValue(_(
             "SNMPPowerDriver: SNMPv3 privacy requires authentication. "
             "Please add `driver_info/auth_protocol` property to node "
             "%(node)s configuration.") % {'node': node.uuid})
 
-    if ('auth_protocol' in snmp_info and
-            'auth_key' not in snmp_info):
+    if ('auth_protocol' in snmp_info
+            and 'auth_key' not in snmp_info):
         raise exception.MissingParameterValue(_(
             "SNMPPowerDriver: missing SNMPv3 authentication key while "
             "`driver_info/snmp_auth_protocol` is present. Please "
             "add `driver_info/snmp_auth_key` to node %(node)s "
             "configuration.") % {'node': node.uuid})
 
-    if ('priv_protocol' in snmp_info and
-            'priv_key' not in snmp_info):
+    if ('priv_protocol' in snmp_info
+            and 'priv_key' not in snmp_info):
         raise exception.MissingParameterValue(_(
             "SNMPPowerDriver: missing SNMPv3 privacy key while "
-            "`driver_info/snmp_priv_protocol` is present. Please"
+            "`driver_info/snmp_priv_protocol` is present. Please "
             "add `driver_info/snmp_priv_key` to node %(node)s "
             "configuration.") % {'node': node.uuid})
 
