@@ -34,7 +34,6 @@ from ironic.common import images
 from ironic.common import swift
 from ironic.conductor import task_manager
 from ironic.conductor import utils as manager_utils
-from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules.ilo import common as ilo_common
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.db import utils as db_utils
@@ -300,9 +299,10 @@ class IloValidateParametersTestCase(BaseIloTest):
 class IloCommonMethodsTestCase(BaseIloTest):
 
     @mock.patch.object(os.path, 'isfile', return_value=True, autospec=True)
-    @mock.patch.object(ilo_client, 'IloClient', spec_set=True,
-                       autospec=True)
-    def _test_get_ilo_object(self, ilo_client_mock, isFile_mock, ca_file=None):
+    def _test_get_ilo_object(self, isFile_mock, ca_file=None):
+        if not mock._is_instance_mock(ilo_client):
+            mock.patch.object(ilo_client, 'IloClient', autospec=True).start()
+        ilo_client_mock = ilo_client.IloClient
         self.info['client_timeout'] = 600
         self.info['client_port'] = 4433
         self.info['ilo_verify_ca'] = ca_file
@@ -320,9 +320,10 @@ class IloCommonMethodsTestCase(BaseIloTest):
         self.assertEqual('ilo_object', returned_ilo_object)
 
     @mock.patch.object(os.path, 'isfile', return_value=True, autospec=True)
-    @mock.patch.object(ilo_client, 'IloClient', spec_set=True,
-                       autospec=True)
-    def test_get_ilo_object_snmp(self, ilo_client_mock, isFile_mock):
+    def test_get_ilo_object_snmp(self, isFile_mock):
+        if not mock._is_instance_mock(ilo_client):
+            mock.patch.object(ilo_client, 'IloClient', autospec=True).start()
+        ilo_client_mock = ilo_client.IloClient
         info = {'auth_user': 'user',
                 'auth_prot_pp': '1234',
                 'auth_priv_pp': '4321',
@@ -433,8 +434,8 @@ class IloCommonMethodsTestCase(BaseIloTest):
         tempfile_mock.return_value = mock_image_file_handle
         self.config(use_web_server_for_images=True, group='ilo')
         deploy_args = {'arg1': 'val1', 'arg2': 'val2'}
-        CONF.deploy.http_url = "http://abc.com/httpboot"
-        CONF.deploy.http_root = "/httpboot"
+        CONF.set_override('http_url', 'http://abc.com/httpboot', 'deploy')
+        CONF.set_override('http_root', '/httpboot', 'deploy')
 
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
@@ -540,7 +541,7 @@ class IloCommonMethodsTestCase(BaseIloTest):
                                                    set_boot_mode_mock):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
-            task.node.instance_info['deploy_boot_mode'] = 'bios'
+            task.node.driver_internal_info['deploy_boot_mode'] = 'bios'
             ilo_common.update_boot_mode(task)
             set_boot_mode_mock.assert_called_once_with(task.node, 'bios')
 
@@ -563,8 +564,8 @@ class IloCommonMethodsTestCase(BaseIloTest):
                                   shared=False) as task:
             ilo_common.update_boot_mode(task)
             set_boot_mode_mock.assert_called_once_with(task.node, 'bios')
-            self.assertEqual('bios',
-                             task.node.instance_info['deploy_boot_mode'])
+            self.assertEqual(
+                'bios', task.node.driver_internal_info['deploy_boot_mode'])
 
     @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
                        autospec=True)
@@ -578,8 +579,8 @@ class IloCommonMethodsTestCase(BaseIloTest):
             ilo_common.update_boot_mode(task)
             get_ilo_object_mock.assert_called_once_with(task.node)
             ilo_mock_obj.get_pending_boot_mode.assert_called_once_with()
-            self.assertEqual('bios',
-                             task.node.instance_info['deploy_boot_mode'])
+            self.assertEqual(
+                'bios', task.node.driver_internal_info['deploy_boot_mode'])
 
     @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
                        autospec=True)
@@ -595,8 +596,8 @@ class IloCommonMethodsTestCase(BaseIloTest):
             get_ilo_object_mock.assert_called_once_with(task.node)
             ilo_mock_obj.get_pending_boot_mode.assert_called_once_with()
             set_pending_boot_mode_mock.assert_called_once_with('UEFI')
-            self.assertEqual('uefi',
-                             task.node.instance_info['deploy_boot_mode'])
+            self.assertEqual(
+                'uefi', task.node.driver_internal_info['deploy_boot_mode'])
 
     @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
                        autospec=True)
@@ -628,8 +629,8 @@ class IloCommonMethodsTestCase(BaseIloTest):
             ilo_common.update_boot_mode(task)
             get_ilo_object_mock.assert_called_once_with(task.node)
             ilo_mock_obj.get_pending_boot_mode.assert_called_once_with()
-            self.assertEqual('bios',
-                             task.node.instance_info['deploy_boot_mode'])
+            self.assertEqual(
+                'bios', task.node.driver_internal_info['deploy_boot_mode'])
 
     @mock.patch.object(ilo_common, 'set_boot_mode', spec_set=True,
                        autospec=True)
@@ -675,8 +676,8 @@ class IloCommonMethodsTestCase(BaseIloTest):
         swift_obj_mock = swift_api_mock.return_value
         boot_iso = 'swift:object-name'
         swift_obj_mock.get_temp_url.return_value = 'image_url'
-        CONF.ilo.swift_ilo_container = 'ilo_cont'
-        CONF.ilo.swift_object_expiry_timeout = 1
+        CONF.set_override('swift_ilo_container', 'ilo_cont', 'ilo')
+        CONF.set_override('swift_object_expiry_timeout', 1, 'ilo')
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             ilo_common.setup_vmedia_for_boot(task, boot_iso)
@@ -703,7 +704,7 @@ class IloCommonMethodsTestCase(BaseIloTest):
     def test_cleanup_vmedia_boot(self, get_name_mock, swift_api_mock,
                                  eject_mock):
         swift_obj_mock = swift_api_mock.return_value
-        CONF.ilo.swift_ilo_container = 'ilo_cont'
+        CONF.set_override('swift_ilo_container', 'ilo_cont', 'ilo')
 
         get_name_mock.return_value = 'image-node-uuid'
 
@@ -726,7 +727,7 @@ class IloCommonMethodsTestCase(BaseIloTest):
         exc = exception.SwiftOperationError('error')
         swift_obj_mock = swift_api_mock.return_value
         swift_obj_mock.delete_object.side_effect = exc
-        CONF.ilo.swift_ilo_container = 'ilo_cont'
+        CONF.set_override('swift_ilo_container', 'ilo_cont', 'ilo')
 
         get_name_mock.return_value = 'image-node-uuid'
 
@@ -751,7 +752,7 @@ class IloCommonMethodsTestCase(BaseIloTest):
         exc = exception.SwiftObjectNotFoundError('error')
         swift_obj_mock = swift_api_mock.return_value
         swift_obj_mock.delete_object.side_effect = exc
-        CONF.ilo.swift_ilo_container = 'ilo_cont'
+        CONF.set_override('swift_ilo_container', 'ilo_cont', 'ilo')
 
         get_name_mock.return_value = 'image-node-uuid'
 
@@ -770,7 +771,7 @@ class IloCommonMethodsTestCase(BaseIloTest):
     def test_cleanup_vmedia_boot_for_webserver(self,
                                                destroy_image_mock,
                                                eject_mock):
-        CONF.ilo.use_web_server_for_images = True
+        CONF.set_override('use_web_server_for_images', True, 'ilo')
 
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
@@ -905,14 +906,34 @@ class IloCommonMethodsTestCase(BaseIloTest):
                               task, False)
         ilo_mock_object.set_secure_boot_mode.assert_called_once_with(False)
 
+    def test_validate_security_parameter_values(self):
+        param_info = {'param': 'password_complexity'}
+        param, enabled, ignored = (
+            ilo_common.validate_security_parameter_values(param_info))
+        self.assertEqual('password_complexity', param)
+        self.assertEqual(False, ignored)
+        self.assertEqual(True, enabled)
+
+    def test_validate_security_parameter_values_no_param(self):
+        param_info = {'param': None}
+        self.assertRaises(exception.MissingParameterValue,
+                          ilo_common.validate_security_parameter_values,
+                          param_info)
+
+    def test_validate_security_parameter_values_invalid_param(self):
+        param_info = {'param': 'minimum_password_length'}
+        self.assertRaises(exception.InvalidParameterValue,
+                          ilo_common.validate_security_parameter_values,
+                          param_info)
+
     @mock.patch.object(os, 'chmod', spec_set=True,
                        autospec=True)
     @mock.patch.object(shutil, 'copyfile', spec_set=True,
                        autospec=True)
     def test_copy_image_to_web_server(self, copy_mock,
                                       chmod_mock):
-        CONF.deploy.http_url = "http://x.y.z.a/webserver/"
-        CONF.deploy.http_root = "/webserver"
+        CONF.set_override('http_url', 'http://x.y.z.a/webserver/', 'deploy')
+        CONF.set_override('http_root', '/webserver', 'deploy')
         expected_url = "http://x.y.z.a/webserver/image-UUID"
         source = 'tmp_image_file'
         destination = "image-UUID"
@@ -928,8 +949,8 @@ class IloCommonMethodsTestCase(BaseIloTest):
                        autospec=True)
     def test_copy_image_to_web_server_fails(self, copy_mock,
                                             chmod_mock):
-        CONF.deploy.http_url = "http://x.y.z.a/webserver/"
-        CONF.deploy.http_root = "/webserver"
+        CONF.set_override('http_url', 'http://x.y.z.a/webserver/', 'deploy')
+        CONF.set_override('http_root', '/webserver', 'deploy')
         source = 'tmp_image_file'
         destination = "image-UUID"
         image_path = "/webserver/image-UUID"
@@ -944,8 +965,8 @@ class IloCommonMethodsTestCase(BaseIloTest):
     @mock.patch.object(ilo_common, 'ironic_utils', autospec=True)
     def test_remove_image_from_web_server(self, utils_mock):
         # | GIVEN |
-        CONF.deploy.http_url = "http://x.y.z.a/webserver/"
-        CONF.deploy.http_root = "/webserver"
+        CONF.set_override('http_url', 'http://x.y.z.a/webserver/', 'deploy')
+        CONF.set_override('http_root', '/webserver', 'deploy')
         object_name = 'tmp_image_file'
         # | WHEN |
         ilo_common.remove_image_from_web_server(object_name)
@@ -1055,7 +1076,7 @@ class IloCommonMethodsTestCase(BaseIloTest):
     def test_destroy_floppy_image_from_web_server(self, get_floppy_name_mock,
                                                   utils_mock):
         get_floppy_name_mock.return_value = 'image-uuid'
-        CONF.deploy.http_root = "/webserver/"
+        CONF.set_override('http_root', '/webserver/', 'deploy')
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             ilo_common.destroy_floppy_image_from_web_server(task.node)
@@ -1094,32 +1115,6 @@ class IloCommonMethodsTestCase(BaseIloTest):
             func_set_boot_device.assert_called_once_with(task,
                                                          boot_devices.CDROM)
 
-    @mock.patch.object(deploy_utils, 'is_secure_boot_requested', spec_set=True,
-                       autospec=True)
-    @mock.patch.object(ilo_common, 'set_secure_boot_mode', spec_set=True,
-                       autospec=True)
-    def test_update_secure_boot_mode_passed_true(self,
-                                                 func_set_secure_boot_mode,
-                                                 func_is_secure_boot_req):
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=False) as task:
-            func_is_secure_boot_req.return_value = True
-            ilo_common.update_secure_boot_mode(task, True)
-            func_set_secure_boot_mode.assert_called_once_with(task, True)
-
-    @mock.patch.object(deploy_utils, 'is_secure_boot_requested', spec_set=True,
-                       autospec=True)
-    @mock.patch.object(ilo_common, 'set_secure_boot_mode', spec_set=True,
-                       autospec=True)
-    def test_update_secure_boot_mode_passed_false(self,
-                                                  func_set_secure_boot_mode,
-                                                  func_is_secure_boot_req):
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=False) as task:
-            func_is_secure_boot_req.return_value = False
-            ilo_common.update_secure_boot_mode(task, False)
-            self.assertFalse(func_set_secure_boot_mode.called)
-
     @mock.patch.object(ironic_utils, 'unlink_without_raise', spec_set=True,
                        autospec=True)
     def test_remove_single_or_list_of_files_with_file_list(self, unlink_mock):
@@ -1221,6 +1216,219 @@ class IloCommonMethodsTestCase(BaseIloTest):
                               task.node)
         ilo_mock_object.get_host_post_state.assert_called_once_with()
 
+    @mock.patch.object(os.path, 'exists', spec_set=True,
+                       autospec=True)
+    def test__get_certificate_file_list_none(self, path_exists_mock):
+        cl = None
+        CONF.set_override('webserver_verify_ca', '/file/path')
+        path_exists_mock.return_value = True
+        expected = ['/file/path']
+        actual = ilo_common._get_certificate_file_list(cl)
+        self.assertEqual(expected, actual)
+
+    @mock.patch.object(os.path, 'exists', spec_set=True,
+                       autospec=True)
+    def test__get_certificate_file_list_empty(self, path_exists_mock):
+        cl = []
+        CONF.set_override('webserver_verify_ca', '/file/path')
+        path_exists_mock.return_value = True
+        expected = ['/file/path']
+        actual = ilo_common._get_certificate_file_list(cl)
+        self.assertEqual(expected, actual)
+
+    @mock.patch.object(os.path, 'exists', spec_set=True,
+                       autospec=True)
+    def test__get_certificate_file_list_empty_no_path(self, path_exists_mock):
+        cl = []
+        CONF.set_override('webserver_verify_ca', '/file/path')
+        path_exists_mock.return_value = False
+        expected = []
+        actual = ilo_common._get_certificate_file_list(cl)
+        self.assertEqual(expected, actual)
+
+    def test__get_certificate_file_list(self):
+        cl = ['file/path/a', 'file/path/b']
+        CONF.set_override('webserver_verify_ca', '/file/path/c')
+        expected = cl
+        actual = ilo_common._get_certificate_file_list(cl)
+        self.assertEqual(expected, actual)
+
+    def test__get_certificate_file_list_string_type(self):
+        cl = 'file/path/a'
+        CONF.set_override('webserver_verify_ca', '/file/path/c')
+        self.assertRaisesRegex(exception.InvalidParameterValue,
+                               "List of files is .* \"<class 'str'>\" .*",
+                               ilo_common._get_certificate_file_list, cl)
+
+    @mock.patch.object(ilo_common, '_get_certificate_file_list', spec_set=True,
+                       autospec=True)
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_add_certificates_false(self, get_ilo_object_mock, get_cl_mock):
+        ilo_mock_object = get_ilo_object_mock.return_value
+        c_l = ['/file/path/a', '/file/path/b']
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            task.node.driver_info['ilo_add_certificates'] = "false"
+            ilo_common.add_certificates(task, c_l)
+
+        get_cl_mock.assert_not_called()
+        ilo_mock_object.add_tls_certificate.assert_not_called()
+
+    @mock.patch.object(ilo_common, '_get_certificate_file_list', spec_set=True,
+                       autospec=True)
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_add_certificates_true(self, get_ilo_object_mock, get_cl_mock):
+        ilo_mock_object = get_ilo_object_mock.return_value
+        c_l = ['/file/path/a', '/file/path/b']
+        get_cl_mock.return_value = c_l
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            task.node.driver_info['ilo_add_certificates'] = "true"
+            ilo_common.add_certificates(task, c_l)
+
+        get_cl_mock.assert_called_once_with(c_l)
+        ilo_mock_object.add_tls_certificate.assert_called_once_with(c_l)
+
+    @mock.patch.object(ilo_common, '_get_certificate_file_list', spec_set=True,
+                       autospec=True)
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_add_certificates_None(self, get_ilo_object_mock, get_cl_mock):
+        ilo_mock_object = get_ilo_object_mock.return_value
+        c_l = ['/file/path/a', '/file/path/b']
+        get_cl_mock.return_value = c_l
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            task.node.driver_info['ilo_add_certificates'] = None
+            ilo_common.add_certificates(task, c_l)
+
+        get_cl_mock.assert_called_once_with(c_l)
+        ilo_mock_object.add_tls_certificate.assert_called_once_with(c_l)
+
+    @mock.patch.object(ilo_common, '_get_certificate_file_list', spec_set=True,
+                       autospec=True)
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_add_certificates_invalid(self, get_ilo_object_mock, get_cl_mock):
+        ilo_mock_object = get_ilo_object_mock.return_value
+        c_l = ['/file/path/a', '/file/path/b']
+        get_cl_mock.return_value = c_l
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            task.node.driver_info['ilo_add_certificates'] = "xyz"
+            self.assertRaisesRegex(exception.InvalidParameterValue,
+                                   "Invalid value type set in driver_info.*",
+                                   ilo_common.add_certificates,
+                                   task, c_l)
+
+        get_cl_mock.assert_not_called()
+        ilo_mock_object.add_tls_certificate.assert_not_called()
+
+    @mock.patch.object(ilo_common, '_get_certificate_file_list', spec_set=True,
+                       autospec=True)
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_add_certificates_true_default(self, get_ilo_object_mock,
+                                           get_cl_mock):
+
+        ilo_mock_object = get_ilo_object_mock.return_value
+        c_l = ['/file/path/A']
+        get_cl_mock.return_value = c_l
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            task.node.driver_info['ilo_add_certificates'] = "true"
+            ilo_common.add_certificates(task)
+
+        get_cl_mock.assert_called_once_with(None)
+        ilo_mock_object.add_tls_certificate.assert_called_once_with(c_l)
+
+    @mock.patch.object(ilo_common, '_get_certificate_file_list', spec_set=True,
+                       autospec=True)
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_add_certificates_raises_ilo_error(self, get_ilo_object_mock,
+                                               get_cl_mock):
+        CONF.set_override('webserver_verify_ca', False)
+        ilo_mock_object = get_ilo_object_mock.return_value
+        c_l = ['/file/path/a', '/file/path/b']
+        get_cl_mock.return_value = c_l
+        exc = ilo_error.IloError('error')
+        ilo_mock_object.add_tls_certificate.side_effect = exc
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            task.node.driver_info['ilo_add_certificates'] = "true"
+            self.assertRaises(exception.IloOperationError,
+                              ilo_common.add_certificates,
+                              task, c_l)
+
+        get_cl_mock.assert_called_once_with(c_l)
+        ilo_mock_object.add_tls_certificate.assert_called_once_with(c_l)
+
+    @mock.patch.object(ilo_common, '_get_certificate_file_list',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_clear_certificates(self, get_ilo_object_mock,
+                                get_certs_mock):
+        ilo_mock_object = get_ilo_object_mock.return_value
+        c_l = ['/file/path/a', '/file/path/b']
+        get_certs_mock.return_value = ['/file/path/x']
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            ilo_common.clear_certificates(task, c_l)
+
+        ilo_mock_object.remove_tls_certificate.assert_called_once_with(
+            cert_file_list=c_l, excl_cert_file_list=['/file/path/x'])
+        get_certs_mock.assert_called_once_with(None)
+
+    @mock.patch.object(ilo_common, '_get_certificate_file_list',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_clear_certificates_default(self, get_ilo_object_mock,
+                                        get_certs_mock):
+        ilo_mock_object = get_ilo_object_mock.return_value
+        get_certs_mock.return_value = ['/file/path/x']
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            ilo_common.clear_certificates(task)
+
+        ilo_mock_object.remove_tls_certificate.assert_called_once_with(
+            cert_file_list=None, excl_cert_file_list=['/file/path/x'])
+        get_certs_mock.assert_called_once_with(None)
+
+    @mock.patch.object(ilo_common, '_get_certificate_file_list',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_clear_certificates_raises_ilo_error(self, get_ilo_object_mock,
+                                                 get_certs_mock):
+        ilo_mock_object = get_ilo_object_mock.return_value
+        c_l = ['/file/path/a', '/file/path/b']
+        get_certs_mock.return_value = ['/file/path/x']
+        exc = ilo_error.IloError('error')
+        ilo_mock_object.remove_tls_certificate.side_effect = exc
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            self.assertRaises(exception.IloOperationError,
+                              ilo_common.clear_certificates,
+                              task, c_l)
+
+        ilo_mock_object.remove_tls_certificate.assert_called_once_with(
+            cert_file_list=c_l, excl_cert_file_list=['/file/path/x'])
+        get_certs_mock.assert_called_once_with(None)
+
     @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
                        autospec=True)
     def test_setup_uefi_https_scheme_http(self, get_ilo_object_mock):
