@@ -382,6 +382,48 @@ class IloManagementTestCase(test_common.BaseIloTest):
                                   **activate_license_args)
                 self.assertFalse(step_mock.called)
 
+    @mock.patch.object(ilo_management, '_execute_ilo_step',
+                       spec_set=True, autospec=True)
+    def test_security_parameters_update(self, step_mock):
+        security_parameters = [
+            {
+                "param": "password_complexity",
+                "enable": True,
+                "ignore": False
+            }
+        ]
+        security_parameters_args = {
+            'security_parameters': security_parameters}
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            task.driver.management.security_parameters_update(
+                task, **security_parameters_args)
+            step_mock.assert_called_once_with(
+                task.node, 'update_password_complexity', True, False)
+
+    @mock.patch.object(ilo_management, '_execute_ilo_step',
+                       spec_set=True, autospec=True)
+    def test_update_minimum_password_length(self, step_mock):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            minimum_password_args = {'password_length': '8'}
+            task.driver.management.update_minimum_password_length(
+                task, **minimum_password_args)
+            step_mock.assert_called_once_with(
+                task.node, 'update_minimum_password_length', '8', False)
+
+    @mock.patch.object(ilo_management, '_execute_ilo_step',
+                       spec_set=True, autospec=True)
+    def test_update_auth_failure_logging_threshold(self, step_mock):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            auth_failure_args = {'logging_threshold': '1'}
+            task.driver.management.update_auth_failure_logging_threshold(
+                task, **auth_failure_args)
+            step_mock.assert_called_once_with(
+                task.node, 'update_authentication_failure_logging', '1', False)
+
     @mock.patch.object(deploy_utils, 'build_agent_options',
                        spec_set=True, autospec=True)
     @mock.patch.object(ilo_boot.IloVirtualMediaBoot, 'clean_up_ramdisk',
@@ -1483,17 +1525,15 @@ class Ilo5ManagementTestCase(db_base.DbTestCase):
     def setUp(self):
         super(Ilo5ManagementTestCase, self).setUp()
         self.driver = mock.Mock(management=ilo_management.Ilo5Management())
-        self.clean_step = {'step': 'erase_devices',
-                           'interface': 'management'}
+
         n = {
             'driver': 'ilo5',
             'driver_info': INFO_DICT,
-            'clean_step': self.clean_step,
         }
         self.config(enabled_hardware_types=['ilo5'],
                     enabled_boot_interfaces=['ilo-virtual-media'],
                     enabled_console_interfaces=['ilo'],
-                    enabled_deploy_interfaces=['iscsi'],
+                    enabled_deploy_interfaces=['direct'],
                     enabled_inspect_interfaces=['ilo'],
                     enabled_management_interfaces=['ilo5'],
                     enabled_power_interfaces=['ilo'],
@@ -1505,6 +1545,9 @@ class Ilo5ManagementTestCase(db_base.DbTestCase):
     @mock.patch.object(ilo_common, 'get_ilo_object', autospec=True)
     @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
     def test_erase_devices_hdd(self, mock_power, ilo_mock, build_agent_mock):
+        self.node.clean_step = {'interface': 'management',
+                                'step': 'erase_devices'}
+        self.node.save()
         ilo_mock_object = ilo_mock.return_value
         ilo_mock_object.get_available_disk_types.return_value = ['HDD']
         build_agent_mock.return_value = []
@@ -1530,6 +1573,9 @@ class Ilo5ManagementTestCase(db_base.DbTestCase):
     @mock.patch.object(ilo_common, 'get_ilo_object', autospec=True)
     @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
     def test_erase_devices_ssd(self, mock_power, ilo_mock, build_agent_mock):
+        self.node.clean_step = {'interface': 'management',
+                                'step': 'erase_devices'}
+        self.node.save()
         ilo_mock_object = ilo_mock.return_value
         ilo_mock_object.get_available_disk_types.return_value = ['SSD']
         build_agent_mock.return_value = []
@@ -1559,6 +1605,9 @@ class Ilo5ManagementTestCase(db_base.DbTestCase):
     @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
     def test_erase_devices_ssd_when_hdd_done(self, mock_power, ilo_mock,
                                              build_agent_mock):
+        self.node.clean_step = {'interface': 'management',
+                                'step': 'erase_devices'}
+        self.node.save()
         build_agent_mock.return_value = []
         ilo_mock_object = ilo_mock.return_value
         ilo_mock_object.get_available_disk_types.return_value = ['HDD', 'SSD']
@@ -1590,6 +1639,9 @@ class Ilo5ManagementTestCase(db_base.DbTestCase):
     @mock.patch.object(ilo_common, 'get_ilo_object', autospec=True)
     def test_erase_devices_completed(self, ilo_mock, disk_status_mock,
                                      log_mock):
+        self.node.clean_step = {'interface': 'management',
+                                'step': 'erase_devices'}
+        self.node.save()
         ilo_mock_object = ilo_mock.return_value
         ilo_mock_object.get_available_disk_types.return_value = ['HDD', 'SSD']
         disk_status_mock.return_value = True
@@ -1613,6 +1665,9 @@ class Ilo5ManagementTestCase(db_base.DbTestCase):
     @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
     def test_erase_devices_hdd_with_erase_pattern_zero(
             self, mock_power, ilo_mock, build_agent_mock):
+        self.node.clean_step = {'interface': 'management',
+                                'step': 'erase_devices'}
+        self.node.save()
         ilo_mock_object = ilo_mock.return_value
         ilo_mock_object.get_available_disk_types.return_value = ['HDD']
         build_agent_mock.return_value = []
@@ -1638,6 +1693,9 @@ class Ilo5ManagementTestCase(db_base.DbTestCase):
     @mock.patch.object(ilo_common, 'get_ilo_object', autospec=True)
     def test_erase_devices_when_no_drive_available(
             self, ilo_mock, log_mock):
+        self.node.clean_step = {'interface': 'management',
+                                'step': 'erase_devices'}
+        self.node.save()
         ilo_mock_object = ilo_mock.return_value
         ilo_mock_object.get_available_disk_types.return_value = []
         with task_manager.acquire(self.context, self.node.uuid,
@@ -1647,6 +1705,9 @@ class Ilo5ManagementTestCase(db_base.DbTestCase):
 
     def test_erase_devices_hdd_with_invalid_format_erase_pattern(
             self):
+        self.node.clean_step = {'interface': 'management',
+                                'step': 'erase_devices'}
+        self.node.save()
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             self.assertRaises(exception.InvalidParameterValue,
@@ -1655,6 +1716,9 @@ class Ilo5ManagementTestCase(db_base.DbTestCase):
 
     def test_erase_devices_hdd_with_invalid_device_type_erase_pattern(
             self):
+        self.node.clean_step = {'interface': 'management',
+                                'step': 'erase_devices'}
+        self.node.save()
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             self.assertRaises(exception.InvalidParameterValue,
@@ -1663,6 +1727,9 @@ class Ilo5ManagementTestCase(db_base.DbTestCase):
 
     def test_erase_devices_hdd_with_invalid_erase_pattern(
             self):
+        self.node.clean_step = {'interface': 'management',
+                                'step': 'erase_devices'}
+        self.node.save()
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             self.assertRaises(exception.InvalidParameterValue,
@@ -1674,6 +1741,9 @@ class Ilo5ManagementTestCase(db_base.DbTestCase):
                        autospec=True)
     def test_erase_devices_hdd_ilo_error(self, clean_err_handler_mock,
                                          ilo_mock):
+        self.node.clean_step = {'interface': 'management',
+                                'step': 'erase_devices'}
+        self.node.save()
         ilo_mock_object = ilo_mock.return_value
         ilo_mock_object.get_available_disk_types.return_value = ['HDD']
         exc = ilo_error.IloError('error')
@@ -1734,3 +1804,131 @@ class Ilo5ManagementTestCase(db_base.DbTestCase):
                 errmsg=exc)
             self.assertTrue(
                 ilo_mock_object.do_one_button_secure_erase.called)
+
+    @mock.patch.object(deploy_utils, 'get_async_step_return_state',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(deploy_utils, 'set_async_step_flags',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(deploy_utils, 'build_agent_options',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
+    @mock.patch.object(ilo_boot.IloVirtualMediaBoot, 'prepare_ramdisk',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(ilo_common, 'clear_certificates', spec_set=True,
+                       autospec=True)
+    def test_clear_ca_certificates(self, clear_certs_mock,
+                                   prepare_ramdisk_mock, node_power_mock,
+                                   build_agent_mock, set_async_mock,
+                                   get_async_ret_mock):
+        self.node.clean_step = {'interface': 'management',
+                                'step': 'clear_ca_certificates'}
+        self.node.save()
+        build_agent_mock.return_value = {'a': 'x', 'b': 'y'}
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+
+            certificate_files = ["/path/to/certsA", "/path/to/certsB"]
+
+            task.driver.management.clear_ca_certificates(
+                task, certificate_files)
+            clear_certs_mock.assert_called_once_with(
+                task, certificate_files)
+            self.assertTrue(task.node.driver_internal_info.get(
+                'clear_ca_certs_flag'))
+            build_agent_mock.assert_called_once_with(task.node)
+            prepare_ramdisk_mock.assert_called_once_with(
+                mock.ANY, task, {'a': 'x', 'b': 'y'})
+            node_power_mock.assert_called_once_with(task, states.REBOOT)
+            set_async_mock.assert_called_once_with(
+                task.node, reboot=True, skip_current_step=False)
+            get_async_ret_mock.assert_called_once_with(task.node)
+
+    @mock.patch.object(deploy_utils, 'get_async_step_return_state',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(deploy_utils, 'set_async_step_flags',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(deploy_utils, 'build_agent_options',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
+    @mock.patch.object(ilo_boot.IloVirtualMediaBoot, 'prepare_ramdisk',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(ilo_common, 'clear_certificates', spec_set=True,
+                       autospec=True)
+    def test_clear_ca_certificates_clear_flag(
+            self, clear_certs_mock, prepare_ramdisk_mock, node_power_mock,
+            build_agent_mock, set_async_mock, get_async_ret_mock):
+        self.node.clean_step = {'interface': 'management',
+                                'step': 'clear_ca_certificates'}
+
+        self.node.save()
+        build_agent_mock.return_value = {'a': 'x', 'b': 'y'}
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+
+            certificate_files = ["/path/to/certsA", "/path/to/certsB"]
+            driver_internal_info = task.node.driver_internal_info
+            driver_internal_info['clear_ca_certs_flag'] = True
+            task.node.driver_internal_info = driver_internal_info
+
+            task.driver.management.clear_ca_certificates(
+                task, certificate_files)
+            clear_certs_mock.assert_not_called()
+
+            build_agent_mock.assert_not_called()
+            prepare_ramdisk_mock.assert_not_called()
+            node_power_mock.assert_not_called()
+            set_async_mock.assert_not_called()
+            get_async_ret_mock.assert_not_called()
+
+    @mock.patch.object(deploy_utils, 'get_async_step_return_state',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(deploy_utils, 'set_async_step_flags',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(deploy_utils, 'build_agent_options',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
+    @mock.patch.object(ilo_boot.IloVirtualMediaBoot, 'prepare_ramdisk',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(ilo_common, 'clear_certificates', spec_set=True,
+                       autospec=True)
+    def test_clear_ca_certificates_ilo_operation_not_supported(
+            self, clear_certs_mock, prepare_ramdisk_mock, node_power_mock,
+            build_agent_mock, set_async_mock, get_async_ret_mock):
+        self.node.clean_step = {'interface': 'management',
+                                'step': 'clear_ca_certificates'}
+        self.node.save()
+        exc = exception.IloOperationNotSupported('error')
+        clear_certs_mock.side_effect = exc
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+
+            certificate_files = ["/path/to/certsA", "/path/to/certsB"]
+
+            self.assertRaises(exception.NodeCleaningFailure,
+                              task.driver.management.clear_ca_certificates,
+                              task, certificate_files)
+            build_agent_mock.assert_not_called()
+            prepare_ramdisk_mock.assert_not_called()
+            node_power_mock.assert_not_called()
+            set_async_mock.assert_not_called()
+            get_async_ret_mock.assert_not_called()
+
+    @mock.patch.object(ilo_common, 'clear_certificates', spec_set=True,
+                       autospec=True)
+    def test_clear_ca_certificates_ilo_operation_error(self, clear_certs_mock):
+        self.node.deploy_step = {'interface': 'management',
+                                 'step': 'clear_ca_certificates'}
+        self.node.save()
+
+        exc = exception.IloOperationError('error')
+        clear_certs_mock.side_effect = exc
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+
+            certificate_files = ["/path/to/certsA", "/path/to/certsB"]
+
+            self.assertRaises(exception.InstanceDeployFailure,
+                              task.driver.management.clear_ca_certificates,
+                              task, certificate_files)
